@@ -9,6 +9,7 @@
 
 #include "base.hh"
 #include <iostream>
+#include <set>
 #include <utility>
 
 #include "charset.hh"
@@ -143,7 +144,7 @@ CMD(db_migrate, "migrate", "", CMD_REF(db), "",
   if (mstat.need_regen())
     {
       database db(app);
-      regenerate_caches(db);
+      regenerate_caches(db, mstat.regen_type());
     }
 
   if (mstat.need_flag_day())
@@ -167,7 +168,12 @@ CMD(db_execute, "execute", "", CMD_REF(db), "",
   db.debug(idx(args, 0)(), cout);
 }
 
-CMD(db_kill_rev_locally, "kill_rev_locally", "", CMD_REF(db), "ID",
+CMD_GROUP(db_local, "local", "", CMD_REF(database),
+          N_("Commands that delete items from the local database"),
+          N_("Deletions cannot be propagated through netsync, so the deleted items "
+             "will come back if you sync with a database that still has them."));
+
+CMD(db_kill_rev_locally, "kill_revision", "", CMD_REF(db_local), "ID",
     N_("Kills a revision from the local database"),
     "",
     options::opts::none)
@@ -239,29 +245,74 @@ CMD(db_kill_rev_locally, "kill_rev_locally", "", CMD_REF(db), "ID",
   db.delete_existing_rev_and_certs(revid);
 }
 
-CMD(db_kill_branch_certs_locally, "kill_branch_certs_locally", "", CMD_REF(db),
-    "BRANCH",
-    N_("Kills branch certificates from the local database"),
-    "",
-    options::opts::none)
+CMD(db_kill_certs_locally, "kill_certs", "", CMD_REF(db_local),
+    "SELECTOR CERTNAME [CERTVAL]",
+    N_("Deletes the specified certs from the local database"),
+    N_("Deletes all certs which are on the given revision(s) and "
+       "have the given name and if a value is specified then also "
+       "the given value."),
+    options::opts::revision)
 {
-  if (args.size() != 1)
+  if (args.size() < 2 || args.size() > 3)
     throw usage(execid);
 
-  database db(app);
-  db.delete_branch_named(typecast_vocab<cert_value>(idx(args, 0)));
-}
-
-CMD(db_kill_tag_locally, "kill_tag_locally", "", CMD_REF(db), "TAG",
-    N_("Kills a tag from the local database"),
-    "",
-    options::opts::none)
-{
-  if (args.size() != 1)
-    throw usage(execid);
+  string selector = idx(args,0)();
+  cert_name name = typecast_vocab<cert_name>(idx(args,1));
 
   database db(app);
-  db.delete_tag_named(typecast_vocab<cert_value>(idx(args, 0)));
+  project_t project(db);
+
+  set<revision_id> revisions;
+  complete(app.opts, app.lua, project, selector, revisions);
+
+
+  transaction_guard guard(db);
+  set<cert_value> branches;
+
+  if (args.size() == 2)
+    {
+      L(FL("deleting all certs named '%s' on %d revisions")
+        % name % revisions.size());
+      for (set<revision_id>::const_iterator r = revisions.begin();
+           r != revisions.end(); ++r)
+        {
+          if (name == branch_cert_name)
+            {
+              vector<cert> to_delete;
+              db.get_revision_certs(*r, name, to_delete);
+              for (vector<cert>::const_iterator i = to_delete.begin();
+                   i != to_delete.end(); ++i)
+                {
+                  branches.insert(i->value);
+                }
+            }
+          db.delete_certs_locally(*r, name);
+        }
+    }
+  else
+    {
+      cert_value value = typecast_vocab<cert_value>(idx(args,2));
+      L(FL("deleting all certs with name '%s' and value '%s' on %d revisions")
+        % name % value % revisions.size());
+      for (set<revision_id>::const_iterator r = revisions.begin();
+           r != revisions.end(); ++r)
+        {
+          db.delete_certs_locally(*r, name, value);
+        }
+      branches.insert(value);
+    }
+
+  for (set<cert_value>::const_iterator i = branches.begin();
+       i != branches.end(); ++i)
+    {
+      db.recalc_branch_leaves(*i);
+      set<revision_id> leaves;
+      db.get_branch_leaves(*i, leaves);
+      if (leaves.empty())
+        db.clear_epoch(typecast_vocab<branch_name>(*i));
+    }
+
+  guard.commit();
 }
 
 CMD(db_check, "check", "", CMD_REF(db), "",
@@ -293,7 +344,7 @@ CMD(db_changesetify, "changesetify", "", CMD_REF(db), "",
   db.check_is_not_rosterified();
 
   // early short-circuit to avoid failure after lots of work
-  cache_user_key(app.opts, app.lua, db, keys, project);
+  cache_user_key(app.opts, project, keys, app.lua);
 
   build_changesets_from_manifest_ancestry(db, keys, project, set<string>());
 }
@@ -301,7 +352,7 @@ CMD(db_changesetify, "changesetify", "", CMD_REF(db), "",
 CMD(db_rosterify, "rosterify", "", CMD_REF(db), "",
     N_("Converts the database to the rosters format"),
     "",
-    options::opts::drop_attr)
+    options::opts::attrs_to_drop)
 {
   database db(app);
   key_store keys(app);
@@ -314,7 +365,7 @@ CMD(db_rosterify, "rosterify", "", CMD_REF(db), "",
   db.check_is_not_rosterified();
 
   // early short-circuit to avoid failure after lots of work
-  cache_user_key(app.opts, app.lua, db, keys, project);
+  cache_user_key(app.opts, project, keys, app.lua);
 
   build_roster_style_revs_from_manifest_style_revs(db, keys, project,
                                                    app.opts.attrs_to_drop);
@@ -329,7 +380,7 @@ CMD(db_regenerate_caches, "regenerate_caches", "", CMD_REF(db), "",
     F("no arguments needed"));
 
   database db(app);
-  regenerate_caches(db);
+  regenerate_caches(db, regen_all);
 }
 
 CMD_HIDDEN(clear_epoch, "clear_epoch", "", CMD_REF(db), "BRANCH",
@@ -493,7 +544,7 @@ CMD(complete, "complete", "", CMD_REF(informative),
     N_("(revision|file|key) PARTIAL-ID"),
     N_("Completes a partial identifier"),
     "",
-    options::opts::verbose)
+    options::opts::none)
 {
   if (args.size() != 2)
     throw usage(execid);
@@ -501,8 +552,6 @@ CMD(complete, "complete", "", CMD_REF(informative),
   database db(app);
   project_t project(db);
 
-  bool verbose = app.opts.verbose;
-
   E(idx(args, 1)().find_first_not_of("abcdef0123456789") == string::npos,
     origin::user,
     F("non-hex digits in partial id"));
@@ -514,7 +563,7 @@ CMD(complete, "complete", "", CMD_REF(informative),
       for (set<revision_id>::const_iterator i = completions.begin();
            i != completions.end(); ++i)
         {
-          if (!verbose) cout << *i << '\n';
+          if (!app.opts.full) cout << *i << '\n';
           else cout << describe_revision(app.opts, app.lua, project, *i) << '\n';
         }
     }
@@ -535,7 +584,7 @@ CMD(complete, "complete", "", CMD_REF(informative),
            i != completions.end(); ++i)
         {
           cout << i->first;
-          if (verbose) cout << ' ' << i->second();
+          if (app.opts.full) cout << ' ' << i->second();
           cout << '\n';
         }
     }
