@@ -1,4 +1,4 @@
-// Copyright (C) 2009 Stephen Leake <stephen_leake@stephe-leake.org>
+// Copyright (C) 2009, 2010 Stephen Leake <stephen_leake@stephe-leake.org>
 // Copyright (C) 2002 Graydon Hoare <graydon@pobox.com>
 //
 // This program is made available under the GNU GPL version 2.0 or
@@ -17,6 +17,7 @@
 #include "asciik.hh"
 #include "charset.hh"
 #include "cmd.hh"
+#include "date_format.hh"
 #include "diff_output.hh"
 #include "file_io.hh"
 #include "parallel_iter.hh"
@@ -144,7 +145,7 @@ dump_diffs(lua_hooks & lua,
               file_path left_path, right_path;
               left_roster.get_name(i.left_key(), left_path);
               // right_path is null
-          
+
               file_id left_id, right_id;
               left_id = downcast_to_file_t(i.left_data())->content;
               // right_id is null
@@ -157,7 +158,7 @@ dump_diffs(lua_hooks & lua,
               if (show_encloser)
                 lua.hook_get_encloser_pattern(left_path, encloser);
 
-              dump_diff(lua, 
+              dump_diff(lua,
                         left_path, right_path,
                         left_id, right_id,
                         left_data, right_data,
@@ -173,7 +174,7 @@ dump_diffs(lua_hooks & lua,
               file_path left_path, right_path;
               // left_path is null
               right_roster.get_name(i.right_key(), right_path);
-           
+
               file_id left_id, right_id;
               // left_id is null
               right_id = downcast_to_file_t(i.right_data())->content;
@@ -186,7 +187,7 @@ dump_diffs(lua_hooks & lua,
               if (show_encloser)
                 lua.hook_get_encloser_pattern(right_path, encloser);
 
-              dump_diff(lua, 
+              dump_diff(lua,
                         left_path, right_path,
                         left_id, right_id,
                         left_data, right_data,
@@ -199,7 +200,7 @@ dump_diffs(lua_hooks & lua,
           // moved/renamed/patched/attribute changes
           if (is_file_t(i.left_data()))
             {
-          
+
               file_id left_id, right_id;
               left_id = downcast_to_file_t(i.left_data())->content;
               right_id = downcast_to_file_t(i.right_data())->content;
@@ -210,7 +211,7 @@ dump_diffs(lua_hooks & lua,
               file_path left_path, right_path;
               left_roster.get_name(i.left_key(), left_path);
               right_roster.get_name(i.right_key(), right_path);
-          
+
               data left_data, right_data;
               get_data(db, left_path, left_id, left_from_db, left_data);
               get_data(db, right_path, right_id, right_from_db, right_data);
@@ -219,7 +220,7 @@ dump_diffs(lua_hooks & lua,
               if (show_encloser)
                 lua.hook_get_encloser_pattern(right_path, encloser);
 
-              dump_diff(lua, 
+              dump_diff(lua,
                         left_path, right_path,
                         left_id, right_id,
                         left_data, right_data,
@@ -250,13 +251,13 @@ prepare_diff(app_state & app,
   // initialize before transaction so we have a database to work with.
   project_t project(db);
 
-  E(app.opts.revision_selectors.size() <= 2, origin::user,
+  E(app.opts.revision.size() <= 2, origin::user,
     F("more than two revisions given"));
 
-  E(!app.opts.reverse || app.opts.revision_selectors.size() == 1, origin::user,
+  E(!app.opts.reverse || app.opts.revision.size() == 1, origin::user,
     F("--reverse only allowed with exactly one revision"));
 
-  if (app.opts.revision_selectors.empty())
+  if (app.opts.revision.empty())
     {
       roster_t left_roster, restricted_roster, right_roster;
       revision_id old_rid;
@@ -275,7 +276,7 @@ prepare_diff(app_state & app,
       work.get_current_roster_shape(db, nis, right_roster);
 
       node_restriction mask(args_to_paths(args),
-                            args_to_paths(app.opts.exclude_patterns),
+                            args_to_paths(app.opts.exclude),
                             app.opts.depth,
                             left_roster, right_roster, ignored_file(work));
 
@@ -292,19 +293,19 @@ prepare_diff(app_state & app,
 
       header << "# old_revision [" << old_rid << "]\n";
     }
-  else if (app.opts.revision_selectors.size() == 1)
+  else if (app.opts.revision.size() == 1)
     {
       roster_t left_roster, restricted_roster, right_roster;
       revision_id r_old_id;
       workspace work(app);
 
-      complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), r_old_id);
+      complete(app.opts, app.lua, project, idx(app.opts.revision, 0)(), r_old_id);
 
       db.get_roster(r_old_id, left_roster);
       work.get_current_roster_shape(db, nis, right_roster);
 
       node_restriction mask(args_to_paths(args),
-                            args_to_paths(app.opts.exclude_patterns),
+                            args_to_paths(app.opts.exclude),
                             app.opts.depth,
                             left_roster, right_roster, ignored_file(work));
 
@@ -330,13 +331,13 @@ prepare_diff(app_state & app,
 
       header << "# old_revision [" << r_old_id << "]\n";
     }
-  else if (app.opts.revision_selectors.size() == 2)
+  else if (app.opts.revision.size() == 2)
     {
       roster_t left_roster, restricted_roster, right_roster;
       revision_id r_old_id, r_new_id;
 
-      complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), r_old_id);
-      complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 1)(), r_new_id);
+      complete(app.opts, app.lua, project, idx(app.opts.revision, 0)(), r_old_id);
+      complete(app.opts, app.lua, project, idx(app.opts.revision, 1)(), r_new_id);
 
       db.get_roster(r_old_id, left_roster);
       db.get_roster(r_new_id, right_roster);
@@ -364,7 +365,7 @@ prepare_diff(app_state & app,
       //   since versioned paths are required to be relative.
 
       node_restriction mask(args_to_paths(args),
-                            args_to_paths(app.opts.exclude_patterns),
+                            args_to_paths(app.opts.exclude),
                             app.opts.depth,
                             left_roster, right_roster);
 
@@ -416,6 +417,10 @@ void dump_header(std::string const & revs,
   out << "#\n";
 }
 
+CMD_PRESET_OPTIONS(diff)
+{
+  opts.with_header = true;
+}
 CMD(diff, "diff", "di", CMD_REF(informative), N_("[PATH]..."),
     N_("Shows current differences"),
     N_("Compares the current tree with the files in the repository and "
@@ -440,7 +445,7 @@ CMD(diff, "diff", "di", CMD_REF(informative), N_("[PATH]..."),
 
   prepare_diff(app, db, old_roster, new_roster, args, old_from_db, new_from_db, revs);
 
-  if (!app.opts.without_header)
+  if (app.opts.with_header)
     {
       dump_header(revs, old_roster, new_roster, cout, true);
     }
@@ -466,9 +471,9 @@ CMD(diff, "diff", "di", CMD_REF(informative), N_("[PATH]..."),
 CMD_AUTOMATE(content_diff, N_("[FILE [...]]"),
              N_("Calculates diffs of files"),
              "",
-             options::opts::with_header | options::opts::without_header |
+             options::opts::au_diff_options |
              options::opts::revision | options::opts::depth |
-             options::opts::exclude | options::opts::reverse)
+             options::opts::exclude)
 {
   roster_t old_roster, new_roster;
   string dummy_header;
@@ -514,7 +519,7 @@ log_certs(vector<cert> const & certs, ostream & os, cert_name const & name,
               I(name == date_cert_name);
               os << date_t(i->value()).as_formatted_localtime(date_fmt);
             }
-            
+
           first = false;
         }
     }
@@ -545,28 +550,98 @@ typedef priority_queue<pair<rev_height, revision_id>,
                        vector<pair<rev_height, revision_id> >,
                        rev_cmp> frontier_t;
 
-CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
-    N_("Prints selected history in forward or reverse order"),
-    N_("This command prints selected history in forward or reverse order, "
-       "filtering it by PATH if given."),
-    options::opts::last | options::opts::next |
-    options::opts::from | options::opts::to | options::opts::revision |
-    options::opts::brief | options::opts::diffs |
-    options::opts::depth | options::opts::exclude |
-    options::opts::no_merges | options::opts::no_files |
-    options::opts::no_graph)
+void
+log_print_rev (app_state &      app,
+               database &       db,
+               project_t &      project,
+               revision_id      rid,
+               revision_t &     rev,
+               string           date_fmt,
+               node_restriction mask,
+               ostream &        out)
+{
+  cert_name const author_name(author_cert_name);
+  cert_name const date_name(date_cert_name);
+  cert_name const branch_name(branch_cert_name);
+  cert_name const tag_name(tag_cert_name);
+  cert_name const changelog_name(changelog_cert_name);
+  cert_name const comment_name(comment_cert_name);
+  vector<cert> certs;
+  project.get_revision_certs(rid, certs);
+
+  if (app.opts.brief)
+    {
+      out << rid;
+      log_certs(certs, out, author_name);
+      if (app.opts.no_graph)
+        log_certs(certs, out, date_name, date_fmt);
+      else
+        {
+          out << '\n';
+          log_certs(certs, out, date_name, date_fmt);
+        }
+      log_certs(certs, out, branch_name);
+      out << '\n';
+    }
+  else
+    {
+      utf8 header;
+      revision_header(rid, rev, certs, date_fmt, header);
+
+      external header_external;
+      utf8_to_system_best_effort(header, header_external);
+      out << header_external;
+
+      if (!app.opts.no_files)
+        {
+          utf8 summary;
+          revision_summary(rev, summary);
+          external summary_external;
+          utf8_to_system_best_effort(summary, summary_external);
+          out << summary_external;
+        }
+    }
+
+  if (app.opts.diffs)
+    {
+      // if the current roster was loaded above this should hit the
+      // cache and not cost much... logging diffs isn't superfast
+      // regardless.
+      roster_t current_roster;
+      db.get_roster(rid, current_roster);
+
+      for (edge_map::const_iterator e = rev.edges.begin();
+           e != rev.edges.end(); ++e)
+        {
+          roster_t parent_roster, restricted_roster;
+
+          db.get_roster(edge_old_revision(e), parent_roster);
+
+          // always show forward diffs from the parent roster to
+          // the current roster regardless of the log direction
+          make_restricted_roster(parent_roster, current_roster,
+                                 restricted_roster, mask);
+
+          dump_diffs(app.lua, db, parent_roster, restricted_roster,
+                     out, app.opts.diff_format,
+                     app.opts.external_diff_args_given,
+                     app.opts.external_diff_args,
+                     true, true,
+                     !app.opts.no_show_encloser);
+        }
+    }
+}
+
+void
+log_common (app_state & app,
+            args_vector args,
+            bool automate,
+            std::ostream & output)
 {
   database db(app);
   project_t project(db);
 
-  string date_fmt;
-  if (app.opts.format_dates)
-    {
-      if (!app.opts.date_fmt.empty())
-        date_fmt = app.opts.date_fmt;
-      else
-        app.lua.hook_get_date_format_spec(date_time_long, date_fmt);
-    }
+  string date_fmt = get_date_format(app.opts, app.lua, date_time_long);
 
   long last = app.opts.last;
   long next = app.opts.next;
@@ -588,7 +663,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
   // start at revisions specified and implied by --from selectors
 
   set<revision_id> starting_revs;
-  if (app.opts.from.empty() && app.opts.revision_selectors.empty())
+  if (app.opts.from.empty() && app.opts.revision.empty())
     {
       // only set default --from revs if no --revision selectors were specified
       workspace work(app, F("try passing a --from revision to start at"));
@@ -656,10 +731,10 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
   // select revisions specified by --revision selectors
 
   set<revision_id> selected_revs;
-  if (!app.opts.revision_selectors.empty())
+  if (!app.opts.revision.empty())
     {
-      for (args_vector::const_iterator i = app.opts.revision_selectors.begin();
-           i != app.opts.revision_selectors.end(); i++)
+      for (args_vector::const_iterator i = app.opts.revision.begin();
+           i != app.opts.revision.end(); i++)
         {
           set<revision_id> rids;
           MM(rids);
@@ -670,16 +745,22 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
           set_difference(rids.begin(), rids.end(),
                          ending_revs.begin(), ending_revs.end(),
                          inserter(selected_revs, selected_revs.end()));
-          if (null_id(first_rid) && i == app.opts.revision_selectors.begin())
+          if (null_id(first_rid) && i == app.opts.revision.begin())
             first_rid = *rids.begin();
         }
     }
 
   L(FL("%d selected revisions") % selected_revs.size());
 
+  // the first restriction mask only includes the actual selected nodes
+  // of the user, so he doesn't get revisions reported in which not the
+  // selected node, but only one of its parents changed
+  // the second restriction mask includes the parent nodes implicitely,
+  // so we can use it to make a restricted roster with it later on
   node_restriction mask;
+  node_restriction mask_diff;
 
-  if (!args.empty() || !app.opts.exclude_patterns.empty())
+  if (!args.empty() || !app.opts.exclude.empty())
     {
       // User wants to trace only specific files
       if (app.opts.from.empty())
@@ -693,9 +774,19 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
           work.get_current_roster_shape(db, nis, new_roster);
 
           mask = node_restriction(args_to_paths(args),
-                                  args_to_paths(app.opts.exclude_patterns),
+                                  args_to_paths(app.opts.exclude),
                                   app.opts.depth, parents, new_roster,
-                                  ignored_file(work));
+                                  ignored_file(work),
+                                  restriction::explicit_includes);
+
+          if (app.opts.diffs)
+            {
+              mask_diff = node_restriction(args_to_paths(args),
+                                           args_to_paths(app.opts.exclude),
+                                           app.opts.depth, parents, new_roster,
+                                           ignored_file(work),
+                                           restriction::implicit_includes);
+            }
         }
       else
         {
@@ -706,13 +797,24 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
           db.get_roster(first_rid, roster);
 
           mask = node_restriction(args_to_paths(args),
-                                  args_to_paths(app.opts.exclude_patterns),
-                                  app.opts.depth, roster);
+                                  args_to_paths(app.opts.exclude),
+                                  app.opts.depth, roster,
+                                  path_always_false<file_path>(),
+                                  restriction::explicit_includes);
+
+          if (app.opts.diffs)
+            {
+              mask_diff = node_restriction(args_to_paths(args),
+                                           args_to_paths(app.opts.exclude),
+                                           app.opts.depth, roster,
+                                           path_always_false<file_path>(),
+                                           restriction::explicit_includes);
+            }
         }
     }
 
   // if --revision was specified without --from log only the selected revs
-  bool log_selected(!app.opts.revision_selectors.empty() &&
+  bool log_selected(!app.opts.revision.empty() &&
                     app.opts.from.empty());
 
   if (log_selected)
@@ -738,12 +840,6 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
       L(FL("log %d starting revisions") % starting_revs.size());
     }
 
-  cert_name const author_name(author_cert_name);
-  cert_name const date_name(date_cert_name);
-  cert_name const branch_name(branch_cert_name);
-  cert_name const tag_name(tag_cert_name);
-  cert_name const changelog_name(changelog_cert_name);
-  cert_name const comment_name(comment_cert_name);
 
   // we can use the markings if we walk backwards for a restricted log
   bool use_markings(direction == log_reverse && !mask.empty());
@@ -751,7 +847,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
   set<revision_id> seen;
   revision_t rev;
   // this is instantiated even when not used, but it's lightweight
-  asciik graph(cout);
+  asciik graph(output);
   while(!frontier.empty() && last != 0 && next != 0)
     {
       revision_id const & rid = frontier.top().second;
@@ -818,7 +914,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
 
       if (app.opts.no_merges && rev.is_merge_node())
         print_this = false;
-      else if (!app.opts.revision_selectors.empty() &&
+      else if (!app.opts.revision.empty() &&
           selected_revs.find(rid) == selected_revs.end())
         print_this = false;
 
@@ -842,89 +938,31 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
 
       if (print_this)
         {
-          vector<cert> certs;
-          project.get_revision_certs(rid, certs);
-
-          ostringstream out;
-          if (app.opts.brief)
-            {
-              out << rid;
-              log_certs(certs, out, author_name);
-              if (app.opts.no_graph)
-                log_certs(certs, out, date_name, date_fmt);
-              else
-                {
-                  out << '\n';
-                  log_certs(certs, out, date_name, date_fmt);
-                }
-              log_certs(certs, out, branch_name);
-              out << '\n';
-            }
+          if (automate)
+            output << rid << "\n";
           else
             {
-              utf8 header;
-              revision_header(rid, rev, certs, date_fmt, header);
+              ostringstream out;
+              log_print_rev (app, db, project, rid, rev, date_fmt, mask_diff, out);
 
-              external header_external;
-              utf8_to_system_best_effort(header, header_external);
-              out << header_external;
-
-              if (!app.opts.no_files)
-                {
-                  utf8 summary;
-                  revision_summary(rev, summary);
-                  external summary_external;
-                  utf8_to_system_best_effort(summary, summary_external);
-                  out << summary_external;
-                }
-            }
-
-          if (app.opts.diffs)
-            {
-              // if the current roster was loaded above this should hit the
-              // cache and not cost much... logging diffs isn't superfast
-              // regardless.
-              roster_t current_roster;
-              db.get_roster(rid, current_roster);
-
-              for (edge_map::const_iterator e = rev.edges.begin();
-                   e != rev.edges.end(); ++e)
-                {
-                  roster_t parent_roster, restricted_roster;
-
-                  db.get_roster(edge_old_revision(e), parent_roster);
-
-                  // always show forward diffs from the parent roster to
-                  // the current roster regardless of the log direction
-                  make_restricted_roster(parent_roster, current_roster,
-                                         restricted_roster, mask);
-
-                  dump_diffs(app.lua, db, parent_roster, restricted_roster,
-                             out, app.opts.diff_format, 
-                             app.opts.external_diff_args_given,
-                             app.opts.external_diff_args,
-                             true, true,
-                             !app.opts.no_show_encloser);
-                }
+              string out_system;
+              utf8_to_system_best_effort(utf8(out.str(), origin::internal), out_system);
+              if (app.opts.no_graph)
+                output << out_system;
+              else
+                graph.print(rid, interesting, out_system);
             }
 
           if (next > 0)
             next--;
           else if (last > 0)
             last--;
-
-          string out_system;
-          utf8_to_system_best_effort(utf8(out.str(), origin::internal), out_system);
-          if (app.opts.no_graph)
-            cout << out_system;
-          else
-            graph.print(rid, interesting, out_system);
         }
-      else if (use_markings && !app.opts.no_graph)
+      else if (!automate && use_markings && !app.opts.no_graph)
         graph.print(rid, interesting,
                     (F("(Revision: %s)") % rid).str());
 
-      cout.flush();
+      output.flush();
 
       frontier.pop(); // beware: rid is invalid from now on
 
@@ -944,6 +982,31 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
     }
 }
 
+CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
+    N_("Prints selected history in forward or reverse order"),
+    N_("This command prints selected history in forward or reverse order, "
+       "filtering it by PATH if given."),
+    options::opts::last | options::opts::next |
+    options::opts::from | options::opts::to | options::opts::revision |
+    options::opts::brief | options::opts::diffs |
+    options::opts::depth | options::opts::exclude |
+    options::opts::no_merges | options::opts::no_files |
+    options::opts::no_graph)
+{
+  log_common (app, args, false, cout);
+}
+
+CMD_AUTOMATE(log, N_("[PATH] ..."),
+             N_("list selected revision history"),
+             "",
+    options::opts::last | options::opts::next |
+    options::opts::from | options::opts::to |
+    options::opts::depth | options::opts::exclude |
+    options::opts::no_merges)
+{
+  log_common (app, args, true, output);
+}
+
 // Local Variables:
 // mode: C++
 // fill-column: 76
