@@ -136,11 +136,28 @@ pick_branch_for_update(options & opts, database & db,
   db.get_revision_certs(chosen_rid, branch_cert_name, certs);
   db.erase_bogus_certs(project, certs);
 
-  set< branch_name > branches;
+  set<branch_name> branches;
   for (vector<cert>::const_iterator i = certs.begin();
        i != certs.end(); i++)
     branches.insert(typecast_vocab<branch_name>(i->value));
 
+  if (!opts.ignore_suspend_certs)
+    {
+      vector<cert> suspend_certs;
+      db.get_revision_certs(chosen_rid, suspend_cert_name, suspend_certs);
+
+      for (vector<cert>::const_iterator i = suspend_certs.begin();
+           i != suspend_certs.end(); i++)
+        {
+          branch_name susp_branch = typecast_vocab<branch_name>(i->value);
+          set<branch_name>::iterator pos = branches.find(susp_branch);
+          if (pos != branches.end())
+            {
+              branches.erase(pos);
+            }
+        }
+    }
+
   if (branches.find(opts.branch) != branches.end())
     {
       L(FL("using existing branch %s") % opts.branch());
@@ -167,7 +184,6 @@ pick_branch_for_update(options & opts, database & db,
         }
       else
         {
-          I(branches.empty());
           W(F("target revision not in any branch\n"
               "next commit will use branch %s")
             % opts.branch);
@@ -201,7 +217,7 @@ update(app_state & app,
     F("cannot determine branch for update"));
 
   revision_id chosen_rid;
-  if (app.opts.revision_selectors.empty())
+  if (app.opts.revision.empty())
     {
       P(F("updating along branch '%s'") % app.opts.branch);
       set<revision_id> candidates;
@@ -228,7 +244,7 @@ update(app_state & app,
     }
   else
     {
-      complete(app.opts, app.lua, project, app.opts.revision_selectors[0](), chosen_rid);
+      complete(app.opts, app.lua, project, app.opts.revision[0](), chosen_rid);
     }
   I(!null_id(chosen_rid));
 
@@ -352,7 +368,7 @@ CMD(update, "update", "", CMD_REF(workspace), "",
   if (!args.empty())
     throw usage(execid);
 
-  if (app.opts.revision_selectors.size() > 1)
+  if (app.opts.revision.size() > 1)
     throw usage(execid);
 
   update(app, args);
@@ -367,7 +383,7 @@ CMD_AUTOMATE(update, "",
   E(args.empty(), origin::user,
     F("wrong argument count"));
 
-  E(app.opts.revision_selectors.size() <= 1, origin::user,
+  E(app.opts.revision.size() <= 1, origin::user,
     F("at most one revision selector may be specified"));
 
   update(app, args);
@@ -497,7 +513,7 @@ CMD(merge, "merge", "", CMD_REF(tree), "",
     "",
     options::opts::branch | options::opts::date | options::opts::author |
     options::opts::messages | options::opts::resolve_conflicts_opts |
-    options::opts::maybe_auto_update)
+    options::opts::auto_update)
 {
   database db(app);
   key_store keys(app);
@@ -527,11 +543,11 @@ CMD(merge, "merge", "", CMD_REF(tree), "",
       % heads.size() % app.opts.branch);
 
   // avoid failure after lots of work
-  cache_user_key(app.opts, app.lua, db, keys, project);
+  cache_user_key(app.opts, project, keys, app.lua);
 
   size_t pass = 1, todo = heads.size() - 1;
 
-  if (app.opts.resolve_conflicts_given || app.opts.resolve_conflicts_file_given)
+  if (app.opts.resolve_conflicts)
     {
       // conflicts and resolutions only apply to first merge, so only do that one.
       todo = 1;
@@ -571,20 +587,6 @@ CMD(merge, "merge", "", CMD_REF(tree), "",
   updater.maybe_do_update();
 }
 
-CMD(propagate, "propagate", "", CMD_REF(tree),
-    N_("SOURCE-BRANCH DEST-BRANCH"),
-    N_("Merges from one branch to another asymmetrically"),
-    "",
-    options::opts::date | options::opts::author | options::opts::messages |
-    options::opts::resolve_conflicts_opts)
-{
-  if (args.size() != 2)
-    throw usage(execid);
-  args_vector a = args;
-  a.push_back(arg_type());
-  process(app, make_command_id("tree merge_into_dir"), a);
-}
-
 //   This is a special merge operator, but very useful for people
 //   maintaining "slightly disparate but related" trees. It does a one-way
 //   merge; less powerful than putting things in the same branch and also
@@ -611,12 +613,9 @@ CMD(propagate, "propagate", "", CMD_REF(tree),
 //   If dir is not the empty string, rename the root of N1 to have the name
 //   'dir' in the merged tree. (ie, it has name "basename(dir)", and its
 //   parent node is "N2.get_node(dirname(dir))")
-CMD(merge_into_dir, "merge_into_dir", "", CMD_REF(tree),
-    N_("SOURCE-BRANCH DEST-BRANCH DIR"),
-    N_("Merges one branch into a subdirectory in another branch"),
-    "",
-    options::opts::date | options::opts::author | options::opts::messages |
-    options::opts::resolve_conflicts_opts | options::opts::maybe_auto_update)
+void perform_merge_into_dir(app_state & app,
+                            commands::command_id const & execid,
+                            args_vector const & args)
 {
   database db(app);
   key_store keys(app);
@@ -654,7 +653,7 @@ CMD(merge_into_dir, "merge_into_dir", "", CMD_REF(tree),
       return;
     }
 
-  cache_user_key(app.opts, app.lua, db, keys, project);
+  cache_user_key(app.opts, project, keys, app.lua);
 
   P(F("propagating %s -> %s") % idx(args,0) % idx(args,1));
   P(F("[left]  %s") % *src_i);
@@ -767,6 +766,30 @@ CMD(merge_into_dir, "merge_into_dir", "", CMD_REF(tree),
   updater.maybe_do_update();
 }
 
+CMD(propagate, "propagate", "", CMD_REF(tree),
+    N_("SOURCE-BRANCH DEST-BRANCH"),
+    N_("Merges from one branch to another asymmetrically"),
+    "",
+    options::opts::date | options::opts::author | options::opts::messages |
+    options::opts::resolve_conflicts_opts)
+{
+  if (args.size() != 2)
+    throw usage(execid);
+  args_vector a = args;
+  a.push_back(arg_type());
+  perform_merge_into_dir(app, make_command_id("tree merge_into_dir"), a);
+}
+
+CMD(merge_into_dir, "merge_into_dir", "", CMD_REF(tree),
+    N_("SOURCE-BRANCH DEST-BRANCH DIR"),
+    N_("Merges one branch into a subdirectory in another branch"),
+    "",
+    options::opts::date | options::opts::author | options::opts::messages |
+    options::opts::resolve_conflicts_opts | options::opts::auto_update)
+{
+  perform_merge_into_dir(app, execid, args);
+}
+
 CMD(merge_into_workspace, "merge_into_workspace", "", CMD_REF(tree),
     N_("OTHER-REVISION"),
     N_("Merges a revision into the current workspace's base revision"),
@@ -891,7 +914,7 @@ CMD(explicit_merge, "explicit_merge", "", CMD_REF(tree),
        "DEST-BRANCH."),
     options::opts::date | options::opts::author |
     options::opts::messages | options::opts::resolve_conflicts_opts |
-    options::opts::maybe_auto_update)
+    options::opts::auto_update)
 {
   database db(app);
   key_store keys(app);
@@ -919,7 +942,7 @@ CMD(explicit_merge, "explicit_merge", "", CMD_REF(tree),
     % right % left);
 
   // avoid failure after lots of work
-  cache_user_key(app.opts, app.lua, db, keys, project);
+  cache_user_key(app.opts, project, keys, app.lua);
   merge_two(app.opts, app.lua, project, keys,
             left, right, branch, string("explicit merge"),
             std::cout, false);
@@ -1126,7 +1149,7 @@ static void get_conflicts_rids(args_vector const & args,
 //   two heads, prints an error message to stderr and exits with status 1.
 //
 CMD_AUTOMATE(show_conflicts, N_("[LEFT_REVID RIGHT_REVID]"),
-             N_("Shows the conflicts between two revisions."),
+             N_("Shows the conflicts between two revisions"),
              N_("If no arguments are given, LEFT_REVID and RIGHT_REVID default to the "
                 "first two heads that would be chosen by the 'merge' command."),
              options::opts::branch | options::opts::ignore_suspend_certs)
@@ -1141,7 +1164,7 @@ CMD_AUTOMATE(show_conflicts, N_("[LEFT_REVID RIGHT_REVID]"),
 
 CMD(store, "store", "", CMD_REF(conflicts),
     "[LEFT_REVID RIGHT_REVID]",
-    N_("Store the conflicts from merging two revisions."),
+    N_("Store the conflicts from merging two revisions"),
     N_("If no arguments are given, LEFT_REVID and RIGHT_REVID default to the "
        "first two heads that would be chosen by the 'merge' command. If "
        "--conflicts-file is not given, '_MTN/conflicts' is used."),
@@ -1151,6 +1174,8 @@ CMD(store, "store", "", CMD_REF(conflicts),
   project_t   project(db);
   revision_id left_id, right_id;
 
+  workspace::require_workspace(F("conflicts file must be under _MTN"));
+
   get_conflicts_rids(args, db, project, app, left_id, right_id);
 
   std::ostringstream output;
@@ -1238,9 +1263,9 @@ CMD(pluck, "pluck", "", CMD_REF(workspace), N_("[-r FROM] -r TO [PATH...]"),
 
   // Work out our arguments
   revision_id from_rid, to_rid;
-  if (app.opts.revision_selectors.size() == 1)
+  if (app.opts.revision.size() == 1)
     {
-      complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), to_rid);
+      complete(app.opts, app.lua, project, idx(app.opts.revision, 0)(), to_rid);
       std::set<revision_id> parents;
       db.get_revision_parents(to_rid, parents);
       E(parents.size() == 1, origin::user,
@@ -1252,10 +1277,10 @@ CMD(pluck, "pluck", "", CMD_REF(workspace), N_("[-r FROM] -r TO [PATH...]"),
         % to_rid);
       from_rid = *parents.begin();
     }
-  else if (app.opts.revision_selectors.size() == 2)
+  else if (app.opts.revision.size() == 2)
     {
-      complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), from_rid);
-      complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 1)(), to_rid);
+      complete(app.opts, app.lua, project, idx(app.opts.revision, 0)(), from_rid);
+      complete(app.opts, app.lua, project, idx(app.opts.revision, 1)(), to_rid);
     }
   else
     throw usage(execid);
@@ -1310,7 +1335,7 @@ CMD(pluck, "pluck", "", CMD_REF(workspace), N_("[-r FROM] -r TO [PATH...]"),
     roster_t to_true_roster;
     db.get_roster(to_rid, to_true_roster);
     node_restriction mask(args_to_paths(args),
-                          args_to_paths(app.opts.exclude_patterns),
+                          args_to_paths(app.opts.exclude),
                           app.opts.depth,
                           *from_roster, to_true_roster,
                           ignored_file(work));
