@@ -32,6 +32,7 @@
 #include "maybe_workspace_updater.hh"
 #include "simplestring_xform.hh"
 #include "database.hh"
+#include "date_format.hh"
 #include "roster.hh"
 #include "rev_output.hh"
 #include "vocab_cast.hh"
@@ -68,8 +69,8 @@ get_old_branch_names(database & db, parent_map const & parents,
 class message_reader
 {
 public:
-  message_reader(string const & message) :
-    message(message), offset(0) {}
+  message_reader(string const & message, size_t offset) :
+    message(message), offset(offset) {}
 
   bool read(string const & text)
   {
@@ -180,73 +181,88 @@ get_log_message_interactively(lua_hooks & lua, workspace & work,
       "You may recover the previous message from this file if necessary."));
 
   utf8 instructions(
-    _("Enter a description of this change following the Changelog line below.\n"
-      "The values of Author, Date and Branch may be modified as required.\n"
-      "\n"));
+    _("-- Enter a description of this change above --\n"
+      "-- You may edit the fields below            --\n"));
+
+  utf8 ignored(
+    _("\n-- Modifications below this line are ignored --\n"));
 
   utf8 cancel(_("*** REMOVE THIS LINE TO CANCEL THE COMMIT ***\n"));
 
+  utf8 const BRANCH(_("Branch:   "));
+  utf8 const AUTHOR(_("Author:   "));
+  utf8 const DATE(  _("Date:     "));
+
+  bool is_date_fmt_valid = date_fmt_valid(date_fmt);
+
   utf8 changelog;
   work.read_user_log(changelog);
 
-  // ensure the changelog message is non-empty so that the Changelog: cert
-  // line is produced by revision_header and there is somewhere to enter a
-  // message
+  // ensure there are two blank lines after the changelog
 
-  string text = changelog();
+  changelog = utf8(changelog() + "\n\n", origin::user);
 
-  if (text.empty() || text[text.length()-1] != '\n')
-    {
-      text += '\n';
-      changelog = utf8(text, origin::user);
-    }
+  // build editable fields
+  utf8 editable;
+  {
+    ostringstream oss;
 
-  ostringstream oss;
+    oss << BRANCH << ' ' << branch << '\n';
+    oss << AUTHOR << ' ' << author << '\n';
 
-  oss << string(70, '-') << '\n';
-  if (!old_branches.empty() && old_branches.find(branch) == old_branches.end())
-    {
-      oss << _("*** THIS REVISION WILL CREATE A NEW BRANCH ***") << "\n\n";
-      for (set<branch_name>::const_iterator i = old_branches.begin();
-           i != old_branches.end(); ++i)
-        oss << _("Old Branch: ") << *i << '\n';
-      oss << _("New Branch: ") << branch << "\n\n";
-    }
-  set<revision_id> heads;
-  project.get_branch_heads(branch, heads, false);
-  if (!heads.empty())
-    {
-      for (edge_map::const_iterator e = rev.edges.begin();
-           e != rev.edges.end(); ++e)
-        {
-          if (heads.find(edge_old_revision(e)) == heads.end())
-            {
-              oss << _("*** THIS REVISION WILL CREATE DIVERGENCE ***") << "\n\n";
-              break;
-            }
-        }
-    }
+    if (!is_date_fmt_valid)
+      {
+        W(F("date format '%s' cannot be parsed; using default instead") % date_fmt);
+      }
 
-  utf8 notes(oss.str().c_str());
+    if (!is_date_fmt_valid || date_fmt.empty())
+      {
+        oss << DATE << ' ' << date << '\n';
+      }
+    else
+      {
+        oss << DATE << ' ' << date.as_formatted_localtime(date_fmt) << '\n';
+      }
+
+    editable = utf8(oss.str().c_str());
+  }
+
+  // Build notes
+  utf8 notes;
+  {
+    ostringstream oss;
+
+    if (!old_branches.empty() && old_branches.find(branch) == old_branches.end())
+      {
+        oss << _("*** THIS REVISION WILL CREATE A NEW BRANCH ***") << "\n\n";
+        for (set<branch_name>::const_iterator i = old_branches.begin();
+             i != old_branches.end(); ++i)
+          oss << _("Old Branch: ") << *i << '\n';
+        oss << _("New Branch: ") << branch << "\n\n";
+      }
+    set<revision_id> heads;
+    project.get_branch_heads(branch, heads, false);
+    if (!heads.empty())
+      {
+        for (edge_map::const_iterator e = rev.edges.begin();
+             e != rev.edges.end(); ++e)
+          {
+            if (heads.find(edge_old_revision(e)) == heads.end())
+              {
+                oss << _("*** THIS REVISION WILL CREATE DIVERGENCE ***") << "\n\n";
+                break;
+              }
+          }
+      }
+
+    notes = utf8(oss.str().c_str());
+  }
 
-  utf8 header;
   utf8 summary;
-
-  bool is_date_fmt_valid = date_fmt_valid(date_fmt);
-  string null_date_fmt("");
-
-  if (!is_date_fmt_valid)
-    {
-      W(F("date format '%s' cannot be used for commit; using default instead") % date_fmt);
-      revision_header(rid, rev, author, date, branch, changelog, null_date_fmt, header);
-    }
-  else
-    {
-      revision_header(rid, rev, author, date, branch, changelog, date_fmt, header);
-    }
   revision_summary(rev, summary);
 
-  utf8 full_message(instructions() + cancel() + header() + notes() + summary(),
+  utf8 full_message(changelog() + cancel() + instructions() + editable() + ignored() +
+                    notes() + summary(),
                     origin::internal);
 
   external input_message;
@@ -260,52 +276,51 @@ get_log_message_interactively(lua_hooks & lua, workspace & work,
 
   system_to_utf8(output_message, full_message);
 
-  // save the message in _MTN/commit so its not lost if something fails below
+  // Everything up to the cancel message is the changelog; trailing blank
+  // lines trimmed.
+  size_t changelog_end = full_message().find(cancel());
+  if (changelog_end == string::npos)
+    {
+      // try to save the edited changelog, now delimited by the instructions.
+      changelog_end = full_message().find(instructions());
+      if (changelog_end != string::npos)
+        work.write_user_log(utf8(trim_right(full_message().substr(0, changelog_end)) + '\n', origin::user));
+
+      E(false, origin::user, F("Commit cancelled."));
+    }
+
+  // save the message in _MTN/commit so it's not lost if something fails
+  // below
   work.save_commit_text(full_message);
 
-  message_reader message(full_message());
+  string content = trim_right(full_message().substr(0, changelog_end)) + '\n';
+  log_message = utf8(content, origin::user);
 
-  // Check the message carefully to make sure the user didn't edit somewhere
-  // outside of the author, date, branch or changelog values. The section
-  // between the "Changelog: " line from the header and the following line
-  // of dashes preceeding the summary is where they should be adding
-  // lines. Ideally, there is a blank line following "Changelog:"
-  // (preceeding the changelog message) and another blank line preceeding
-  // the next line of dashes (following the changelog message) but both of
-  // these are optional.
+  message_reader message(full_message(), changelog_end);
+
+  // Parse the editable fields.
+
+  // this can't fail, since we start reading where we found it above.
+  message.read(cancel());
 
   E(message.read(instructions()), origin::user,
     F("Commit failed. Instructions not found."));
 
-  if (!message.read(cancel()))
-    {
-      // clear the backup file if the commit was explicitly cancelled
-      work.clear_commit_text();
-      E(message.read(cancel()), origin::user,
-        F("Commit cancelled."));
-    }
+  // Branch:
 
-  utf8 const AUTHOR(trim_right(_("Author: ")).c_str());
-  utf8 const DATE(trim_right(_("Date: ")).c_str());
-  utf8 const BRANCH(trim_right(_("Branch: ")).c_str());
-  utf8 const CHANGELOG(trim_right(_("Changelog: ")).c_str());
+  E(message.read(trim_right(BRANCH())), origin::user,
+    F("Commit failed. Branch header not found."));
 
-  // ----------------------------------------------------------------------
-  // Revision:
-  // Parent:
-  // Parent:
-  // Author:
+  string b = message.readline();
 
-  size_t pos = header().find(AUTHOR()); // look in unedited header
-  I(pos != string::npos);
+  E(!b.empty(), origin::user,
+    F("Commit failed. Branch value empty."));
 
-  string prefix = header().substr(0, pos);
-  E(message.read(prefix), origin::user,
-    F("Commit failed. Revision/Parent header not found."));
+  branch = branch_name(b, origin::user);
 
   // Author:
 
-  E(message.read(AUTHOR()), origin::user,
+  E(message.read(trim_right(AUTHOR())), origin::user,
     F("Commit failed. Author header not found."));
 
   author = message.readline();
@@ -315,7 +330,7 @@ get_log_message_interactively(lua_hooks & lua, workspace & work,
 
   // Date:
 
-  E(message.read(DATE()), origin::user,
+  E(message.read(trim_right(DATE())), origin::user,
     F("Commit failed. Date header not found."));
 
   string d = message.readline();
@@ -328,43 +343,7 @@ get_log_message_interactively(lua_hooks & lua, workspace & work,
   else
     date = date_t::from_formatted_localtime(d, date_fmt);
 
-  // Branch:
-
-  E(message.read(BRANCH()), origin::user,
-    F("Commit failed. Branch header not found."));
-
-  string b = message.readline();
-
-  E(!b.empty(), origin::user,
-    F("Commit failed. Branch value empty."));
-
-  branch = branch_name(b, origin::user);
-
-  string blank = message.readline();
-  E(blank == "", origin::user,
-    F("Commit failed. Blank line before Changelog header not found."));
-
-  // Changelog:
-
-  E(message.read(CHANGELOG()), origin::user,
-    F("Commit failed. Changelog header not found."));
-
-  // remove the summary before extracting the changelog content
-
-  string footer = notes() + summary();
-
-  if (!footer.empty())
-    {
-      E(message.contains(footer), origin::user,
-        F("Commit failed. Change summary not found."));
-
-      E(message.remove(footer), origin::user,
-        F("Commit failed. Text following Change summary."));
-    }
-
-  string content = trim(message.content()) + '\n';
-
-  log_message = utf8(content, origin::user);
+  // rest is ignored
 
   // remove the backup file now that all values have been extracted
   work.clear_commit_text();
@@ -378,7 +357,7 @@ revert(app_state & app,
   roster_t old_roster, new_roster;
   cset preserved;
 
-  E(app.opts.missing || !args.empty() || !app.opts.exclude_patterns.empty(),
+  E(app.opts.missing || !args.empty() || !app.opts.exclude.empty(),
     origin::user,
     F("you must pass at least one path to 'revert' (perhaps '.')"));
 
@@ -397,7 +376,7 @@ revert(app_state & app,
   }
 
   node_restriction mask(args_to_paths(args),
-                        args_to_paths(app.opts.exclude_patterns),
+                        args_to_paths(app.opts.exclude),
                         app.opts.depth,
                         old_roster, new_roster, ignored_file(work),
                         restriction::explicit_includes);
@@ -574,43 +553,115 @@ CMD(undrop, "undrop", "", CMD_REF(workspace), N_("[PATH]..."),
   revert(app, args, true);
 }
 
-CMD(disapprove, "disapprove", "", CMD_REF(review), N_("REVISION"),
-    N_("Disapproves a particular revision"),
+static void
+walk_revisions(database & db, const revision_id & from_rev,
+               const revision_id & to_rev)
+{
+  revision_id r = from_rev;
+  revision_t rev;
+
+  do
+    {
+      E(!null_id(r), origin::user,
+        F("revision %s it not a child of %s, cannot invert")
+        % from_rev % to_rev);
+
+      db.get_revision(r, rev);
+      E(rev.edges.size() < 2, origin::user,
+        F("revision %s has %d parents, cannot invert")
+        % r % rev.edges.size());
+
+      E(rev.edges.size() > 0, origin::user,
+        F("revision %s it not a child of %s, cannot invert")
+        % from_rev % to_rev);
+      r = edge_old_revision (rev.edges.begin());
+    }
+  while (r != to_rev);
+}
+
+CMD(disapprove, "disapprove", "", CMD_REF(review),
+    N_("[PARENT-REVISION] CHILD-REVISION"),
+    N_("Disapproves a particular revision or revision range"),
     "",
     options::opts::branch | options::opts::messages | options::opts::date |
-    options::opts::author | options::opts::maybe_auto_update)
+    options::opts::author | options::opts::auto_update)
 {
   database db(app);
   key_store keys(app);
   project_t project(db);
 
-  if (args.size() != 1)
+  if (args.size() < 1 || args.size() > 2)
     throw usage(execid);
 
   maybe_workspace_updater updater(app, project);
 
   utf8 log_message("");
   bool log_message_given;
-  revision_id r;
+  revision_id child_rev, parent_rev;
   revision_t rev, rev_inverse;
   shared_ptr<cset> cs_inverse(new cset());
-  complete(app.opts, app.lua, project, idx(args, 0)(), r);
-  db.get_revision(r, rev);
 
-  E(rev.edges.size() == 1, origin::user,
-    F("revision %s has %d changesets, cannot invert")
-      % r % rev.edges.size());
+  if (args.size() == 1)
+    {
+      complete(app.opts, app.lua, project, idx(args, 0)(), child_rev);
+      db.get_revision(child_rev, rev);
 
-  guess_branch(app.opts, project, r);
-  E(!app.opts.branch().empty(), origin::user,
-    F("need --branch argument for disapproval"));
+      E(rev.edges.size() == 1, origin::user,
+        F("revision %s has %d parents, cannot invert")
+        % child_rev % rev.edges.size());
 
-  process_commit_message_args(app.opts, log_message_given, log_message,
-                              utf8((FL("disapproval of revision '%s'")
-                                    % r).str(),
-                                   origin::internal));
+      guess_branch(app.opts, project, child_rev);
+      E(!app.opts.branch().empty(), origin::user,
+        F("need --branch argument for disapproval"));
 
-  cache_user_key(app.opts, app.lua, db, keys, project);
+      process_commit_message_args(app.opts, log_message_given, log_message,
+                                  utf8((FL("disapproval of revision '%s'")
+                                        % child_rev).str(),
+                                       origin::internal));
+    }
+  else if (args.size() == 2)
+    {
+      complete(app.opts, app.lua, project, idx(args, 0)(), parent_rev);
+      complete(app.opts, app.lua, project, idx(args, 1)(), child_rev);
+
+      set<revision_id> rev_set;
+
+      rev_set.insert(child_rev);
+      rev_set.insert(parent_rev);
+
+      erase_ancestors(db, rev_set);
+      if (rev_set.size() > 1)
+        {
+          set<revision_id> ancestors;
+          db.get_common_ancestors (rev_set, ancestors);
+          E(ancestors.size() > 0, origin::user,
+            F("revisions %s and %s do not share common history, cannot invert")
+            % parent_rev % child_rev);
+          E(ancestors.size() < 1, origin::user,
+            F("revisions share common history"
+              ", but %s is not an ancestor of %s, cannot invert")
+            % parent_rev % child_rev);
+        }
+
+      walk_revisions(db, child_rev, parent_rev);
+      db.get_revision(parent_rev, rev);
+
+      E(rev.edges.size() == 1, origin::user,
+        F("revision %s has %d parents, cannot invert")
+        % child_rev % rev.edges.size());
+
+      guess_branch(app.opts, project, child_rev);
+      E(!app.opts.branch().empty(), origin::user,
+        F("need --branch argument for disapproval"));
+
+      process_commit_message_args(app.opts, log_message_given, log_message,
+                                  utf8((FL("disapproval of revisions "
+                                           "'%s'..'%s'")
+                                        % parent_rev % child_rev).str(),
+                                       origin::internal));
+    }
+
+  cache_user_key(app.opts, project, keys, app.lua);
 
   // for the divergence check, below
   set<revision_id> heads;
@@ -624,10 +675,10 @@ CMD(disapprove, "disapprove", "", CMD_REF(review), N_("REVISION"),
   {
     roster_t old_roster, new_roster;
     db.get_roster(edge_old_revision(old_edge), old_roster);
-    db.get_roster(r, new_roster);
+    db.get_roster(child_rev, new_roster);
     make_cset(new_roster, old_roster, *cs_inverse);
   }
-  rev_inverse.edges.insert(make_pair(r, cs_inverse));
+  rev_inverse.edges.insert(make_pair(child_rev, cs_inverse));
 
   {
     transaction_guard guard(db);
@@ -694,25 +745,16 @@ CMD(mkdir, "mkdir", "", CMD_REF(workspace), N_("[DIRECTORY...]"),
   work.perform_additions(db, paths, false, !app.opts.no_ignore);
 }
 
-CMD(add, "add", "", CMD_REF(workspace), N_("[PATH]..."),
-    N_("Adds files to the workspace"),
-    "",
-    options::opts::unknown | options::opts::no_ignore |
-    options::opts::recursive)
+void perform_add(app_state & app,
+                 database & db,
+                 workspace & work,
+                 vector<file_path> roots)
 {
-  if (!app.opts.unknown && (args.size() < 1))
-    throw usage(execid);
-
-  database db(app);
-  workspace work(app);
-
-  vector<file_path> roots = args_to_paths(args);
-
   set<file_path> paths;
   bool add_recursive = app.opts.recursive;
   if (app.opts.unknown)
     {
-      path_restriction mask(roots, args_to_paths(app.opts.exclude_patterns),
+      path_restriction mask(roots, args_to_paths(app.opts.exclude),
                             app.opts.depth, ignored_file(work));
       set<file_path> ignored;
 
@@ -731,6 +773,48 @@ CMD(add, "add", "", CMD_REF(workspace), N_("[PATH]..."),
   work.perform_additions(db, paths, add_recursive, !app.opts.no_ignore);
 }
 
+CMD(add, "add", "", CMD_REF(workspace), N_("[PATH]..."),
+    N_("Adds files to the workspace"),
+    "",
+    options::opts::unknown | options::opts::no_ignore |
+    options::opts::recursive)
+{
+  if (!app.opts.unknown && (args.size() < 1))
+    throw usage(execid);
+
+  database db(app);
+  workspace work(app);
+
+  vector<file_path> roots = args_to_paths(args);
+
+  perform_add(app, db, work, roots);
+}
+
+void perform_drop(app_state & app,
+                  database & db,
+                  workspace & work,
+                  vector<file_path> roots)
+{
+  set<file_path> paths;
+  if (app.opts.missing)
+    {
+      temp_node_id_source nis;
+      roster_t current_roster_shape;
+      work.get_current_roster_shape(db, nis, current_roster_shape);
+      node_restriction mask(roots,
+                            args_to_paths(app.opts.exclude),
+                            app.opts.depth,
+                            current_roster_shape, ignored_file(work));
+      work.find_missing(current_roster_shape, mask, paths);
+    }
+  else
+    {
+      paths = set<file_path>(roots.begin(), roots.end());
+    }
+
+  work.perform_deletions(db, paths,
+                             app.opts.recursive, app.opts.bookkeep_only);
+}
 CMD(drop, "drop", "rm", CMD_REF(workspace), N_("[PATH]..."),
     N_("Drops files from the workspace"),
     "",
@@ -742,26 +826,7 @@ CMD(drop, "drop", "rm", CMD_REF(workspace), N_("[PATH]..."),
   database db(app);
   workspace work(app);
 
-  set<file_path> paths;
-  if (app.opts.missing)
-    {
-      temp_node_id_source nis;
-      roster_t current_roster_shape;
-      work.get_current_roster_shape(db, nis, current_roster_shape);
-      node_restriction mask(args_to_paths(args),
-                            args_to_paths(app.opts.exclude_patterns),
-                            app.opts.depth,
-                            current_roster_shape, ignored_file(work));
-      work.find_missing(current_roster_shape, mask, paths);
-    }
-  else
-    {
-      vector<file_path> roots = args_to_paths(args);
-      paths = set<file_path>(roots.begin(), roots.end());
-    }
-
-  work.perform_deletions(db, paths,
-                             app.opts.recursive, app.opts.bookkeep_only);
+  perform_drop(app, db, work, args_to_paths(args));
 }
 
 
@@ -836,23 +901,13 @@ CMD(status, "status", "", CMD_REF(informative), N_("[PATH]..."),
   project_t project(db);
   workspace work(app);
 
-  string date_fmt;
-  if (app.opts.format_dates)
-    {
-      if (!app.opts.date_fmt.empty())
-        date_fmt = app.opts.date_fmt;
-      else
-        app.lua.hook_get_date_format_spec(date_time_long, date_fmt);
-    }
-
-  if (!date_fmt_valid(date_fmt))
-    W(F("date format '%s' cannot be used for commit") % date_fmt);
+  string date_fmt = get_date_format(app.opts, app.lua, date_time_long);
 
   work.get_parent_rosters(db, old_rosters);
   work.get_current_roster_shape(db, nis, new_roster);
 
   node_restriction mask(args_to_paths(args),
-                        args_to_paths(app.opts.exclude_patterns),
+                        args_to_paths(app.opts.exclude),
                         app.opts.depth,
                         old_rosters, new_roster, ignored_file(work));
 
@@ -951,13 +1006,9 @@ CMD(status, "status", "", CMD_REF(informative), N_("[PATH]..."),
   cout << summary_external;
 }
 
-CMD(checkout, "checkout", "co", CMD_REF(tree), N_("[DIRECTORY]"),
-    N_("Checks out a revision from the database into a directory"),
-    N_("If a revision is given, that's the one that will be checked out.  "
-       "Otherwise, it will be the head of the branch (given or implicit).  "
-       "If no directory is given, the branch name will be used as directory."),
-    options::opts::branch | options::opts::revision |
-    options::opts::move_conflicting_paths)
+static void
+checkout_common(app_state & app,
+                args_vector const & args)
 {
   revision_id revid;
   system_path dir;
@@ -966,10 +1017,7 @@ CMD(checkout, "checkout", "co", CMD_REF(tree), N_("[DIRECTORY]"),
   project_t project(db);
   transaction_guard guard(db, false);
 
-  if (args.size() > 1 || app.opts.revision_selectors.size() > 1)
-    throw usage(execid);
-
-  if (app.opts.revision_selectors.empty())
+  if (app.opts.revision.empty())
     {
       // use branch head revision
       E(!app.opts.branch().empty(), origin::user,
@@ -992,10 +1040,10 @@ CMD(checkout, "checkout", "co", CMD_REF(tree), N_("[DIRECTORY]"),
         }
       revid = *(heads.begin());
     }
-  else if (app.opts.revision_selectors.size() == 1)
+  else if (app.opts.revision.size() == 1)
     {
       // use specified revision
-      complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), revid);
+      complete(app.opts, app.lua, project, idx(app.opts.revision, 0)(), revid);
 
       guess_branch(app.opts, project, revid);
 
@@ -1060,6 +1108,37 @@ CMD(checkout, "checkout", "co", CMD_REF(tree), N_("[DIRECTORY]"),
   guard.commit();
 }
 
+CMD(checkout, "checkout", "co", CMD_REF(tree), N_("[DIRECTORY]"),
+    N_("Checks out a revision from the database into a directory"),
+    N_("If a revision is given, that's the one that will be checked out.  "
+       "Otherwise, it will be the head of the branch (given or implicit).  "
+       "If no directory is given, the branch name will be used as directory."),
+    options::opts::branch | options::opts::revision |
+    options::opts::move_conflicting_paths)
+{
+  if (args.size() > 1 || app.opts.revision.size() > 1)
+    throw usage(execid);
+
+  checkout_common(app, args);
+}
+
+CMD_AUTOMATE(checkout, N_("[DIRECTORY]"),
+    N_("Checks out a revision from the database into a directory"),
+    N_("If a revision is given, that's the one that will be checked out.  "
+       "Otherwise, it will be the head of the branch (given or implicit).  "
+       "If no directory is given, the branch name will be used as directory."),
+    options::opts::branch | options::opts::revision |
+    options::opts::move_conflicting_paths)
+{
+  E(args.size() < 2, origin::user,
+    F("wrong argument count"));
+
+  E(app.opts.revision.size() < 2, origin::user,
+    F("wrong revision count"));
+
+  checkout_common(app, args);
+}
+
 CMD_GROUP(attr, "attr", "", CMD_REF(workspace),
           N_("Manages file attributes"),
           N_("This command is used to set, get or drop file attributes."));
@@ -1400,18 +1479,14 @@ CMD_AUTOMATE(drop_attribute, N_("PATH [KEY]"),
 
   drop_attr(app, args);
 }
-
-CMD(commit, "commit", "ci", CMD_REF(workspace), N_("[PATH]..."),
-    N_("Commits workspace changes to the database"),
-    "",
-    options::opts::branch | options::opts::message | options::opts::msgfile |
-    options::opts::date | options::opts::author | options::opts::depth |
-    options::opts::exclude)
+void perform_commit(app_state & app,
+                    database & db,
+                    workspace & work,
+                    project_t & project,
+                    commands::command_id const & execid,
+                    vector<file_path> const & paths)
 {
-  database db(app);
   key_store keys(app);
-  workspace work(app);
-  project_t project(db);
 
   utf8 log_message("");
   bool log_message_given;
@@ -1421,20 +1496,13 @@ CMD(commit, "commit", "ci", CMD_REF(workspace), N_("[PATH]..."),
   temp_node_id_source nis;
   cset excluded;
 
-  string date_fmt;
-  if (app.opts.format_dates)
-    {
-      if (!app.opts.date_fmt.empty())
-        date_fmt = app.opts.date_fmt;
-      else
-        app.lua.hook_get_date_format_spec(date_time_long, date_fmt);
-    }
+  string date_fmt = get_date_format(app.opts, app.lua, date_time_long);
 
   work.get_parent_rosters(db, old_rosters);
   work.get_current_roster_shape(db, nis, new_roster);
 
-  node_restriction mask(args_to_paths(args),
-                        args_to_paths(app.opts.exclude_patterns),
+  node_restriction mask(paths,
+                        args_to_paths(app.opts.exclude),
                         app.opts.depth,
                         old_rosters, new_roster, ignored_file(work));
 
@@ -1473,6 +1541,20 @@ CMD(commit, "commit", "ci", CMD_REF(workspace), N_("[PATH]..."),
       app.opts.branch = branchname;
     }
 
+  // now that we have an (unedited) branch name, let the hook decide if the
+  // changes should get committed at all
+  revision_data rev_data;
+  write_revision(restricted_rev, rev_data);
+
+  bool changes_validated;
+  string reason;
+
+  app.lua.hook_validate_changes(rev_data, app.opts.branch,
+                                changes_validated, reason);
+
+  E(changes_validated, origin::user,
+    F("changes rejected by hook: %s") % reason);
+
   if (global_sanity.debug_p())
     {
       L(FL("new manifest '%s'\n"
@@ -1545,17 +1627,14 @@ CMD(commit, "commit", "ci", CMD_REF(workspace), N_("[PATH]..."),
 
   // If the hook doesn't exist, allow the message to be used.
   bool message_validated;
-  string reason, new_manifest_text;
+  reason.clear();
 
-  revision_data new_rev;
-  write_revision(restricted_rev, new_rev);
-
-  app.lua.hook_validate_commit_message(log_message, new_rev, app.opts.branch,
+  app.lua.hook_validate_commit_message(log_message, rev_data, app.opts.branch,
                                        message_validated, reason);
   E(message_validated, origin::user,
     F("log message rejected by hook: %s") % reason);
 
-  cache_user_key(app.opts, app.lua, db, keys, project);
+  cache_user_key(app.opts, project, keys, app.lua);
 
   // for the divergence check, below
   set<revision_id> heads;
@@ -1715,6 +1794,30 @@ CMD(commit, "commit", "ci", CMD_REF(workspace), N_("[PATH]..."),
   }
 }
 
+CMD_PRESET_OPTIONS(commit)
+{
+  // Dates are never parseable on Win32 (see win32/parse_date.cc),
+  // so don't warn about that, just use the default format.
+#ifdef WIN32
+  opts.no_format_dates = true;
+#else
+  opts.no_format_dates = false;
+#endif
+}
+
+CMD(commit, "commit", "ci", CMD_REF(workspace), N_("[PATH]..."),
+    N_("Commits workspace changes to the database"),
+    "",
+    options::opts::branch | options::opts::messages |
+    options::opts::date | options::opts::author | options::opts::depth |
+    options::opts::exclude)
+{
+  database db(app);
+  workspace work(app);
+  project_t project(db);
+  perform_commit(app, db, work, project, execid, args_to_paths(args));
+}
+
 CMD_NO_WORKSPACE(setup, "setup", "", CMD_REF(tree), N_("[DIRECTORY]"),
     N_("Sets up a new workspace directory"),
     N_("If no directory is specified, uses the current directory."),
@@ -1766,7 +1869,7 @@ CMD_NO_WORKSPACE(import, "import", "", CMD_REF(tree), N_("DIRECTORY"),
   N_("Imports the contents of a directory into a branch"),
   "",
   options::opts::branch | options::opts::revision |
-  options::opts::message | options::opts::msgfile |
+  options::opts::messages |
   options::opts::dryrun |
   options::opts::no_ignore | options::opts::exclude |
   options::opts::author | options::opts::date)
@@ -1779,10 +1882,10 @@ CMD_NO_WORKSPACE(import, "import", "", CMD_REF(tree), N_("DIRECTORY"),
   E(args.size() == 1, origin::user,
     F("you must specify a directory to import"));
 
-  if (app.opts.revision_selectors.size() == 1)
+  if (app.opts.revision.size() == 1)
     {
       // use specified revision
-      complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), ident);
+      complete(app.opts, app.lua, project, idx(app.opts.revision, 0)(), ident);
 
       guess_branch(app.opts, project, ident);
 
@@ -1822,48 +1925,53 @@ CMD_NO_WORKSPACE(import, "import", "", CMD_REF(tree), N_("DIRECTORY"),
      F("import directory '%s' doesn't exists") % dir,
      F("import directory '%s' is a file") % dir);
 
+  system_path _MTN_dir = dir / path_component("_MTN");
+
+  require_path_is_nonexistent
+    (_MTN_dir, F("bookkeeping directory already exists in '%s'") % dir);
+
+  directory_cleanup_helper remove_on_fail(_MTN_dir);
+
   workspace::create_workspace(app.opts, app.lua, dir);
   workspace work(app);
 
-  try
+  revision_t rev;
+  make_revision_for_workspace(ident, cset(), rev);
+  work.put_work_rev(rev);
+
+  // prepare stuff for 'add' and so on.
+  options save_opts;
+  // add --unknown
+  save_opts.exclude = app.opts.exclude;
+  app.opts.exclude = args_vector();
+  app.opts.unknown = true;
+  app.opts.recursive = true;
+  perform_add(app, db, work, vector<file_path>());
+  app.opts.recursive = false;
+  app.opts.unknown = false;
+  app.opts.exclude = save_opts.exclude;
+
+  // drop --missing
+  save_opts.no_ignore = app.opts.no_ignore;
+  app.opts.missing = true;
+  perform_drop(app, db, work, vector<file_path>());
+  app.opts.missing = false;
+  app.opts.no_ignore = save_opts.no_ignore;
+
+  // commit
+  if (!app.opts.dryrun)
     {
-      revision_t rev;
-      make_revision_for_workspace(ident, cset(), rev);
-      work.put_work_rev(rev);
-
-      // prepare stuff for 'add' and so on.
-      args_vector empty_args;
-      options save_opts;
-      // add --unknown
-      save_opts.exclude_patterns = app.opts.exclude_patterns;
-      app.opts.exclude_patterns = args_vector();
-      app.opts.unknown = true;
-      app.opts.recursive = true;
-      process(app, make_command_id("workspace add"), empty_args);
-      app.opts.recursive = false;
-      app.opts.unknown = false;
-      app.opts.exclude_patterns = save_opts.exclude_patterns;
-
-      // drop --missing
-      save_opts.no_ignore = app.opts.no_ignore;
-      app.opts.missing = true;
-      process(app, make_command_id("workspace drop"), empty_args);
-      app.opts.missing = false;
-      app.opts.no_ignore = save_opts.no_ignore;
-
-      // commit
-      if (!app.opts.dryrun)
-        process(app, make_command_id("workspace commit"), empty_args);
+        perform_commit(app, db, work, project,
+                       make_command_id("workspace commit"),
+                       vector<file_path>());
+      remove_on_fail.commit();
     }
-  catch (...)
+  else
     {
-      // clean up before rethrowing
-      delete_dir_recursive(bookkeeping_root);
-      throw;
+      // since the _MTN directory gets removed, don't try to write out
+      // _MTN/options at the end
+      workspace::used = false;
     }
-
-  // clean up
-  delete_dir_recursive(bookkeeping_root);
 }
 
 CMD_NO_WORKSPACE(migrate_workspace, "migrate_workspace", "", CMD_REF(tree),
@@ -2174,11 +2282,11 @@ bisect_update(app_state & app, bisect::type type)
   set<revision_id> marked_ids;
 
   // mark the current or specified revisions as good, bad or skipped
-  if (app.opts.revision_selectors.empty())
+  if (app.opts.revision.empty())
     marked_ids.insert(current_id);
   else
-    for (args_vector::const_iterator i = app.opts.revision_selectors.begin();
-         i != app.opts.revision_selectors.end(); i++)
+    for (args_vector::const_iterator i = app.opts.revision.begin();
+         i != app.opts.revision.end(); i++)
       {
         set<revision_id> rids;
         MM(rids);
@@ -2258,8 +2366,8 @@ bisect_update(app_state & app, bisect::type type)
 
 CMD(bisect_status, "status", "", CMD_REF(bisect), "",
     N_("Reports on the current status of the bisection search"),
-    N_("Lists the total number of revisions in the search set; "
-       "the number of revisions that have been determined to be good or bad; "
+    N_("Lists the total number of revisions in the search set, "
+       "the number of revisions that have been determined to be good or bad, "
        "the number of revisions that have been skipped "
        "and the number of revisions remaining to be tested."),
     options::opts::none)
