@@ -1206,13 +1206,13 @@ function get_netsync_connect_command(uri, args)
                 table.insert(argv, "-")
                 table.insert(argv, "UNIX-CONNECT:" .. uri["path"])
         else
-            -- start remote monotone process
             if argv then
+                    -- start remote monotone process
 
                     table.insert(argv, get_mtn_command(uri["host"]))
 
                     if args["debug"] then
-                            table.insert(argv, "--debug")
+                            table.insert(argv, "--verbose")
                     else
                             table.insert(argv, "--quiet")
                     end
@@ -1223,6 +1223,8 @@ function get_netsync_connect_command(uri, args)
                     table.insert(argv, "--stdio")
                     table.insert(argv, "--no-transport-auth")
 
+            -- else scheme does not require starting a new remote
+            -- process (ie mtn:)
             end
         end
         return argv
@@ -1315,8 +1317,11 @@ do
    --   startup			Corresponds to note_mtn_startup()
    --   start			Corresponds to note_netsync_start()
    --   revision_received	Corresponds to note_netsync_revision_received()
+   --   revision_sent		Corresponds to note_netsync_revision_sent()
    --   cert_received		Corresponds to note_netsync_cert_received()
+   --   cert_sent		Corresponds to note_netsync_cert_sent()
    --   pubkey_received		Corresponds to note_netsync_pubkey_received()
+   --   pubkey_sent		Corresponds to note_netsync_pubkey_sent()
    --   end			Corresponds to note_netsync_end()
    --
    -- Those functions take exactly the same arguments as the corresponding
@@ -1462,3 +1467,23 @@ function validate_git_author(author)
 
    return false
 end
+
+function get_man_page_formatter_command()
+   local term_width = guess_terminal_width() - 2
+   -- The string returned is run in a process created with 'popen'
+   -- (see cmd.cc manpage).
+   --
+   -- On Unix (and POSIX compliant systems), 'popen' runs 'sh' with
+   -- the inherited path.
+   --
+   -- On MinGW, 'popen' runs 'cmd.exe' with the inherited path. MinGW
+   -- does not (currently) provide nroff or equivalent. So we assume
+   -- sh, nroff and less are also installed, from Cygwin or some other
+   -- toolset.
+   if string.sub(get_ostype(), 1, 7) == "Windows" then
+      return string.format("sh -c 'nroff -man -rLL=%dn' | less -R", term_width)
+   else
+      return string.format("nroff -man -rLL=%dn | less -R", term_width)
+   end
+end
+
