@@ -2,10 +2,10 @@
 #define CONFIG_H 1
 
 #define PACKAGE "monotone"
-#define PACKAGE_STRING "monotone 0.48"
+#define PACKAGE_STRING "monotone 0.99.1"
 #define PACKAGE_BUGREPORT "monotone-devel@nongnu.org"
 #define LC_MESSAGES LC_CTYPE
-#define VERSION "0.48"
+#define VERSION "0.99.1"
 
 #ifdef _MSC_VER
 typedef unsigned long pid_t;
