@@ -509,7 +509,7 @@ read_options_file(any_path const & optspath,
         }
       else if (opt == "key")
         {
-          opts.signing_key = external_key_name(val, origin::workspace);
+          opts.key = external_key_name(val, origin::workspace);
           opts.key_given = true;
         }
       else if (opt == "keydir")
@@ -544,8 +544,8 @@ write_options_file(bookkeeping_path const & optspath,
 
   if (!opts.branch().empty())
     st.push_str_pair(symbol("branch"), opts.branch());
-  if (!opts.signing_key().empty())
-    st.push_str_pair(symbol("key"), opts.signing_key());
+  if (!opts.key().empty())
+    st.push_str_pair(symbol("key"), opts.key());
   if (!opts.key_dir.as_internal().empty())
     st.push_str_pair(symbol("keydir"), opts.key_dir.as_internal());
 
@@ -584,7 +584,14 @@ workspace::get_options(options & opts)
   if (!opts.key_dir_given && !opts.conf_dir_given && cur_opts.key_dir_given)
     { // if empty/missing, we want to keep the default
       opts.key_dir = cur_opts.key_dir;
-      opts.key_dir_given = true;
+      // one would expect that we should set the key_dir_given flag here, but
+      // we do not because of the interaction between --confdir and --keydir.
+      // If --keydir is not given and --confdir is, then --keydir will default
+      // to the "keys" subdirectory of the given confdir. This works by the 
+      // --confdir option body looking at key_dir_given; if reading the keydir 
+      // from _MTN/options set that, then --confdir would stop setting the 
+      // default keydir when in a workspace. 
+      //opts.key_dir_given = true;
     }
 
   if (opts.branch().empty() && cur_opts.branch_given)
@@ -596,7 +603,7 @@ workspace::get_options(options & opts)
   L(FL("branch name is '%s'") % opts.branch);
 
   if (!opts.key_given)
-    opts.signing_key = cur_opts.signing_key;
+    opts.key = cur_opts.key;
 }
 
 void
@@ -683,9 +690,9 @@ workspace::set_options(options const & opts, lua_hooks & lua, bool branch_is_sti
       options_changed = true;
     }
 
-  if (opts.key_given && cur_opts.signing_key != opts.signing_key)
+  if (opts.key_given && cur_opts.key != opts.key)
     {
-      cur_opts.signing_key = opts.signing_key;
+      cur_opts.key = opts.key;
       options_changed = true;
     }
 
@@ -713,7 +720,7 @@ workspace::print_option(utf8 const & opt, std::ostream & output)
   else if (opt() == "branch")
     output << opts.branch << '\n';
   else if (opt() == "key")
-    output << opts.signing_key << '\n';
+    output << opts.key << '\n';
   else if (opt() == "keydir")
     output << opts.key_dir << '\n';
   else
@@ -1952,7 +1959,7 @@ workspace::perform_rename(database & db,
   // source does not exist as a file.
   if (srcs.size() == 1 && !new_roster.has_node(dst))
     {
-      // "rename SRC DST" case
+      // "rename SRC DST", DST is a file
       file_path const & src = *srcs.begin();
       file_path dpath = dst;
 
@@ -1962,28 +1969,43 @@ workspace::perform_rename(database & db,
       E(new_roster.has_node(src), origin::user,
         F("source file %s is not versioned") % src);
 
-      //this allows the 'magic add' of a non-versioned directory to happen in
-      //all cases.  previously, mtn mv fileA dir/ woudl fail if dir/ wasn't
-      //versioned whereas mtn mv fileA dir/fileA would add dir/ if necessary
-      //and then reparent fileA.
-      if (get_path_status(dst) == path::directory)
-        dpath = dst / src.basename();
+      if (src == dst || dst.is_beneath_of(src))
+        {
+          if (get_path_status(dst) == path::directory)
+            W(F("cannot move `%s' to a subdirectory of itself, `%s/%s'") % src % dst % src);
+          else
+            W(F("`%s' and `%s' are the same file") % src % dst);
+        }
       else
         {
+          //this allows the 'magic add' of a non-versioned directory to happen in
+          //all cases.  previously, mtn mv fileA dir/ woudl fail if dir/ wasn't
+          //versioned whereas mtn mv fileA dir/fileA would add dir/ if necessary
+          //and then reparent fileA.
+          //
+          //Note that we checked above that dst is not a directory
+
           //this handles the case where:
           // touch foo
           // mtn mv foo bar/foo where bar doesn't exist
           file_path parent = dst.dirname();
           E(get_path_status(parent) == path::directory, origin::user,
             F("destination path's parent directory %s/ doesn't exist") % parent);
-        }
 
-      renames.insert(make_pair(src, dpath));
-      add_parent_dirs(db, nis, *this, dpath, new_roster);
+          renames.insert(make_pair(src, dpath));
+          add_parent_dirs(db, nis, *this, dpath, new_roster);
+        }
     }
   else
     {
-      // "rename SRC1 [SRC2 ...] DSTDIR" case
+      // Either srcs has more than one element, or dst is an existing
+      // directory (or both). So we have one of:
+      //
+      // 1) rename SRC1 [SRC2 ...] DSTDIR
+      //
+      // 2) mv foo bar
+      //    mtn mv --bookkeep-only foo bar
+
       E(get_path_status(dst) == path::directory, origin::user,
         F("destination %s/ is not a directory") % dst);
 
@@ -1997,12 +2019,34 @@ workspace::perform_rename(database & db,
             F("source file %s is not versioned") % *i);
 
           file_path d = dst / i->basename();
-          E(!new_roster.has_node(d), origin::user,
-            F("destination %s already exists in the workspace manifest") % d);
+          if (bookkeep_only &&
+              srcs.size() == 1 &&
+              get_path_status(*srcs.begin()) == path::directory &&
+              get_path_status(dst) == path::directory)
+            {
+              // case 2)
+              d = dst;
+            }
+          else
+            {
+              // case 1)
+              d = dst / i->basename();
 
-          renames.insert(make_pair(*i, d));
+              E(!new_roster.has_node(d), origin::user,
+                F("destination %s already exists in the workspace manifest") % d);
+            }
 
-          add_parent_dirs(db, nis, *this, d, new_roster);
+          if (*i == dst || dst.is_beneath_of(*i))
+            {
+              W(F("cannot move `%s' to a subdirectory of itself, `%s/%s'")
+                % *i % dst % *i);
+            }
+          else
+            {
+              renames.insert(make_pair(*i, d));
+
+              add_parent_dirs(db, nis, *this, d, new_roster);
+            }
         }
     }
 
